#Code based on http://blog.dazoot.ro/2009/09/procesare-bounce-uri-in-real-time/
#assumed to be public domain

import base64
import email
import re

acre = re.compile(r'<(?P<addr>[^>]*)>')


class BounceDetectionException(Exception):
    pass
 
class NoDeliveryStatusException(BounceDetectionException):
    pass
 
class WrongDeliveryStatusException(BounceDetectionException):
    pass
   
class BounceParser:   
    email_address = None
    def __init__(self, msg, custom_error_messages):
        self.matched_msg = None
        self.msg = msg
        self.parse()
        self.custom_error_messages = custom_error_messages
   
    def parse(self):
        self._parsed = False
       
        self.status = None
        self.diagnostic_message = None
        self.action = None

       
        self.dsmsg = None
        for part in self.msg.get_payload():
            if type(part) is str:
                raise NoDeliveryStatusException("No message/delivery-status part found.")
            if part.get_content_type() == "message/delivery-status":
                self.dsmsg = part
                break
        if self.dsmsg is None:
            raise NoDeliveryStatusException("No message/delivery-status part found.")

        if not self.dsmsg.is_multipart():
            self.status = self.dsmsg.get("Status", None)
            self.diagnostic_message = self.dsmsg.get("Diagnostic-Code", None)
            self.action = self.dsmsg.get("Action", None)
        else:
            for part in self.dsmsg.get_payload():

                #EEA hack, some messages are attached as files but the Python API for emails will not read that file as another message.
                #import pdb; pdb.set_trace()
                if not part.keys(): 
                    text = part.get_payload()
                    text = base64.b64decode(text)
                    part = email.message_from_string(text)

                    if part.has_key('Original-Recipient'):
                        match = acre.match(part.get('Original-Recipient'))
                        self.email_address = match.group('addr')
                #end EEA hack
                    
                if part.has_key("Status"):
                    self.status = part.get("Status")
                if part.has_key("Diagnostic-Code"):
                    self.diagnostic_message = part.get("Diagnostic-Code", None)

                if part.has_key("Action"):
                    self.action = part.get("Action", None)
       
        if self.status is None: #this status is not that important; the EEA Email server receives delivery failures with no status
            self.status = self.diagnostic_message
        if self.status is None:
            raise WrongDeliveryStatusException("No Status: found in message/delivery-status part.")
        if self.action is None:
            raise WrongDeliveryStatusException("No Action: found in message/delivery-status part.")
        if self.diagnostic_message is not None:
            self.diagnostic_message = self.diagnostic_message.replace("\r", "").replace("\t", " ").replace("\n", " ")
            ## try to get an better email address
            if not self.email_address:
                match = acre.match(self.diagnostic_message)
                if match:
                    self.email_address = match.group('addr')
       
        self._parsed = True
    
    def isParsed(self):
        return self._parsed
 
    def isFailed(self):
        return self.action.lower() == "failed"
    
    def is5xx(self):
        return self.status.find("5.") != 0
   
    def is4xx(self):
        return self.status.find("4.") == 0
   
    def getDiagnosticMessage(self):
        return self.diagnostic_message
   
    def isNoUserFound(self):
        if not self.isFailed():
            return False
       
        if not self.is5xx():
            return False
       
        if "5.1.1" in self.status:
            # @see <a href="http://rfc.sunsite.dk/rfc/rfc2034.html">http://rfc.sunsite.dk/rfc/rfc2034.html</a>
            # 5.1.1 is always no such user / mailbox
            return True
       
        if self.diagnostic_message is None:
            # Missing diagnostig message =&gt; no can do parse :D
            return False
           
        diagnostic_message = self.diagnostic_message.lower().strip()
        for m in self.custom_error_messages:
            if type(m) is str:
                if diagnostic_message.find(m.lower()) != -1:
                    self.matched_msg = m
                    return True
            else:
                if m.search(diagnostic_message):
                    return True

        if self.isFailed():
            return True
       
        return False
   
    def getAction(self):
        return self.action
       
    def getStatus(self):
        return self.status
       
    def getMatchedMsg(self):
        return self.matched_msg


